package ai.pai.lensman.service;

import android.app.Service;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;

import com.amap.api.location.AMapLocation;
import com.amap.api.location.AMapLocationClient;
import com.amap.api.location.AMapLocationClientOption;
import com.amap.api.location.AMapLocationListener;
import com.android.common.utils.LogHelper;



public class MyLocationService extends Service implements Handler.Callback{

    private AMapLocationClient locationClient = null;
    private AMapLocationClientOption locationOption = new AMapLocationClientOption();

    public static final int COMMAND_START_LOCATION = 9801;
    public static final int COMMAND_DESTROY_LOCATION = 9803;

    private static final int MSG_FETCH_TOUR_INFO = 4567;
    private static final int MSG_START_LOCATION = 4568;

    private Handler mHandler;

    private static final String TAG = "定位";

    @Override
    public void onCreate() {
        super.onCreate();
        LogHelper.d(TAG,"服务启动");
        mHandler = new Handler(this);
    }

    @Override
    public IBinder onBind(Intent intent) {
        return null;
    }

    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        if(intent == null){
            return super.onStartCommand(intent,flags,startId);
        }
        Bundle bundle= intent.getExtras();
        if(bundle==null){
            return super.onStartCommand(intent, flags, startId);
        }
        int command = bundle.getInt("command",0);
        initLocation();
        mHandler.sendEmptyMessageDelayed(MSG_FETCH_TOUR_INFO,5000);
        switch (command) {
            case COMMAND_START_LOCATION:
                startLocation();
                break;
            case COMMAND_DESTROY_LOCATION:
                destroyLocation();
                break;
        }
        return START_STICKY;
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        destroyLocation();
    }


    private void initLocation(){
        if(locationClient!=null){
            return;
        }
        locationClient = new AMapLocationClient(getApplicationContext());
        locationClient.setLocationOption(getDefaultOption());
        locationClient.setLocationListener(locationListener);
    }



    private AMapLocationClientOption getDefaultOption(){
        AMapLocationClientOption mOption = new AMapLocationClientOption();
        mOption.setLocationMode(AMapLocationClientOption.AMapLocationMode.Hight_Accuracy);//可选，设置定位模式，可选的模式有高精度、仅设备、仅网络。默认为高精度模式
        mOption.setGpsFirst(false);//可选，设置是否gps优先，只在高精度模式下有效。默认关闭
        mOption.setHttpTimeOut(30000);//可选，设置网络请求超时时间。默认为30秒。在仅设备模式下无效
        mOption.setInterval(30*1000);//可选，设置定位间隔。默认为30秒
        mOption.setNeedAddress(true);//可选，设置是否返回逆地理地址信息。默认是true
        mOption.setOnceLocation(true);//可选，设置是否单次定位。默认是false
        mOption.setOnceLocationLatest(false);//可选，设置是否等待wifi刷新，默认为false.如果设置为true,会自动变为单次定位，持续定位时不要使用
        AMapLocationClientOption.setLocationProtocol(AMapLocationClientOption.AMapLocationProtocol.HTTP);//可选， 设置网络请求的协议。可选HTTP或者HTTPS。默认为HTTP
        mOption.setSensorEnable(false);//可选，设置是否使用传感器。默认是false
        mOption.setWifiScan(true); //可选，设置是否开启wifi扫描。默认为true，如果设置为false会同时停止主动刷新，停止以后完全依赖于系统刷新，定位位置可能存在误差
        return mOption;
    }

    AMapLocationListener locationListener = new AMapLocationListener() {
        @Override
        public void onLocationChanged(AMapLocation loc) {
            if(loc.getErrorCode()==AMapLocation.ERROR_CODE_FAILURE_LOCATION_PERMISSION){
                return;
            }
            double lat = loc.getLatitude();
            double lon = loc.getLongitude();
            LogHelper.d(TAG,"上报经纬度 lat = "+lat+"lon="+lon+" 位置 ="+loc.getAddress());
        }
    };


    private void startLocation(){
        initLocation();
        LogHelper.d(TAG,"当前是旅行模式，开始定位");
        locationClient.setLocationOption(locationOption);
        locationClient.startLocation();
    }


    private void destroyLocation(){
        if (null != locationClient) {
            locationClient.stopLocation();
            locationClient.onDestroy();
            locationClient = null;
            locationOption = null;
        }
        mHandler.removeCallbacksAndMessages(null);
    }

    @Override
    public boolean handleMessage(Message msg) {
        if(msg.what == MSG_FETCH_TOUR_INFO){
            return true;
        }else if(msg.what == MSG_START_LOCATION){
            startLocation();
            return true;
        }
        return false;
    }
}
